package Screens;
import java.util.List;

import com.example.cells.Graphics.Graphics;
import com.example.cells.IO.Touch.TouchEvent;

import Game.Assets;
import Game.Environment;
import Game.Game;
import android.util.Log;

public class GameScreen extends Screen{
	
	enum GameState{
		READY,
		RUNNING,
		PAUSED,
		GAMEOVER
	}
	private GameState state = GameState.READY;
	private Environment environment;
	private int oldScore =0;
	private String score = "0";
	private String TAG="GameScreen: ";
	private StringBuilder builder = new StringBuilder();
	private int selectorPadding = 10; //px of padding for 'warm' selection
	private int radius = 32/2; //later need to add feature for hi-res, mid-res, low-res which will change this val which is for rendering only
	
	public GameScreen(Game game){
		super(game);
		Graphics graphics = game.getGraphics();
		environment = new Environment(graphics.getWidth(), graphics.getHeight());
		//for now instantiate into state of running, but later will add buttons for state changes
		state = GameState.RUNNING;
	}
	
	//main control method distributes to other control methods
	@Override
	public void update(float deltaTime) {
		List<TouchEvent> touchEvents = game.getUserInput();
		if(state==GameState.READY){
			updateReady(touchEvents);
		}
		if(state==GameState.RUNNING){
			updateRunning(touchEvents, deltaTime);
		}
		if(state==GameState.PAUSED){
			updatePaused(touchEvents);
		}
		if(state==GameState.GAMEOVER){
			updateGameOver(touchEvents);
		}
	}
	
	private void updateReady(List<TouchEvent> touchEvents){
		Log.d(TAG, "updateReady()");
	}
	
	//contains controller code for mvc (i.e. chains to environment/cell controllers
	private void updateRunning(List<TouchEvent> touchEvents, float deltaTime){
		Log.d(TAG, "updateRunning()");
		int len = touchEvents.size();
		for (int i=0; i<len; i++){
			TouchEvent event = touchEvents.get(i);
			builder.append("TouchEvent.X: " +(event.x)).append(" TouchEvent.Y: "+event.y ).append("\n");
			Log.d("From TouchEvents BUFFER", builder.toString()+"\n");
			this.checkCellSelector(event);
			
		}
				
			
			//environment.setSelectedCell(event.x, event.y);
			//environment.setSelectedSingularity(event.x, event.y);
	
	environment.update(deltaTime);
	}
	
	private void updatePaused(List<TouchEvent> touchEvents){
		
	}
	
	private void updateGameOver(List<TouchEvent> touchEvents){
		
	}
	
	private void drawEnvironment(Environment environment){
		Graphics graphics = game.getGraphics();
		graphics.drawBitmapHolder(Assets.environmentBackground, 0, 0);
		if(environment.cellSelected){
			graphics.drawBitmapHolder(Assets.cellHealthySelected, environment.CELL_X_POS-radius, environment.CELL_Y_POS-radius);
		}else{
			graphics.drawBitmapHolder(Assets.cellHealthy, environment.CELL_X_POS-radius, environment.CELL_Y_POS-radius);	
		}
		StringBuilder here = new StringBuilder();
		here.append("environment.CELL_X_POS: ").append(environment.CELL_X_POS).append(" environment.CELL_Y_POS: ").append(environment.CELL_Y_POS);
		Log.d("Cell POSITION WITHIN ENVIRONMENT", here.toString());
	//	Log.d(TAG, "CELL_Y_POS: " + builder.append(environment.CELL_Y_POS).toString()+"\n");
		
	}
	
	private void checkCellSelector(TouchEvent event){
		if(!environment.cellSelected){
			if(event.x > environment.CELL_X_POS-radius || event.x < environment.CELL_X_POS+ radius){
				if(event.y > environment.CELL_Y_POS-radius || event.y < environment.CELL_Y_POS+ radius){
					environment.cellSelected = true;
				}
			}
		}else if(environment.cellSelected){
			if(event.x < environment.CELL_X_POS-radius || event.x > environment.CELL_X_POS+ radius){
				environment.cellSelected = false;
				this.changeCellDestination(event.x, event.y);
			}
		}
	}
	
	public void changeCellDestination(int xDestination, int yDestination){
		environment.setSingularity(xDestination, yDestination);
	}

	@Override
	public void present(float deltaTime) {
		Log.d(TAG, "present(float deltaTime)");
		Log.d(TAG, "drawEnvironment() call");
		drawEnvironment(this.environment);
		
	}

	@Override
	public void pause() {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void resume() {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void dispose() {
		// TODO Auto-generated method stub
		
	}
}
