package com.example.cells.Graphics;
import android.content.res.AssetManager;
import android.graphics.Bitmap;
import android.graphics.Bitmap.Config;
import android.graphics.BitmapFactory;
import android.graphics.BitmapFactory.Options;
import android.graphics.Canvas;
import android.graphics.Paint;
import android.graphics.Paint.Style;
import android.graphics.Rect;
import java.io.InputStream;
import java.io.IOException;
import android.util.Log;

/**
 * ARGB888: [32-bit] 24-bit RGB triplet, 8 unused bits specify transparency
 * ARGB444: [16-bit short] 4 bits per color component 
 * RGB565: [16-bit] short, no alpha-blending; more shades of green visible
 * to human eye.
 * Check out chapter 3 p.101 in Android Games book.  
 * This is *sort of* a factory class for generating BitmapHolders from assets
 * @author meghan
 * 
 */

public class Graphics {
	private final static String TAG = "Graphics Class: ";
	private AssetManager assets;
	private Bitmap bitmap;
	public Canvas canvas;
	private Config config;
	private Paint paint;
	private Rect srcRect = new Rect();
	private Rect destRect = new Rect();
	private int width, height;
		
	public Graphics(AssetManager assets, int width, int height){
		Log.d(TAG, "Graphics constructor");
		this.assets = assets;
		this.canvas = new Canvas();
		this.width = width;
		this.height = height;
		//startScreen bitmap to be passed in and set above?
		//this.bitmap = bitmap;
		//this.canvas = new Canvas(bitmap);
		this.paint = new Paint();
		this.config = Config.RGB_565;//default to RGB_565
	}
	
	public void setBitmapConfig(android.graphics.Bitmap.Config config){
		this.config = config;
	}
	
	public Config getBitmapConfig(){
		return this.config;
	}
	
	public BitmapHolder generateBitmap(String fileLocation){
		Options options = new Options(); //BitmapFactory.Options
		options.inPreferredConfig = this.config;
		InputStream inputStream = null;
		Bitmap bitmap = null;
		
		try{
			inputStream = assets.open(fileLocation);
			bitmap = BitmapFactory.decodeStream(inputStream, null, options);
		
			if(bitmap == null){
				throw new RuntimeException("Couldn't load bitmap from asset  "+ fileLocation);
			}
		}catch(IOException e){
			String tag = e.toString();
			Log.d(tag, "Error opening assets or decoding BitmapFactory stream");
		}finally{
			if(inputStream !=null){
				try{inputStream.close();}
				catch(IOException e){
					String tag = e.toString();
					Log.d(tag, "Error closing inputStream");
				}
			}
		}
		return new BitmapHolder(bitmap, options.inPreferredConfig);
	}//generateBitmap
	
	public void drawBitmapHolder(BitmapHolder bitmapHolder, int x, int y){
		float a = (float)x;
		canvas.drawBitmap(bitmapHolder.bitmap, (float)x, (float)y, this.paint);
	}
	
	public void drawBitmapHolder(BitmapHolder bitmapHolder, int x, int y, int srcX, int srcY, int srcWidth, int srcHeight){
		srcRect.left = srcX;
		srcRect.top = srcY;
		srcRect.right = srcX+ srcWidth -1;
		srcRect.bottom = srcY + srcHeight -1;
		
		destRect.left = x;
		destRect.top = y;
		destRect.right = x + srcWidth -1;
		destRect.bottom = y + srcHeight -1;
		
		canvas.drawBitmap(bitmapHolder.bitmap, srcRect, destRect, paint);
	}
	
	public void drawRect(int x, int y, int width, int height, int color){
		paint.setColor(color);
		paint.setStyle(Style.FILL);
		canvas.drawRect(x, y, x+width-1, y+height-1, paint);
	}
	
	public void clear(int color){
		canvas.drawRGB((color & 0xff0000)>>16, (color & 0xff00)>>8, (color & 0xff));
		//0xff0000 = 255.0.0 shift 16 bits    0xf00 = 255.0 shift 8  0xff=255, no shift
	}
	
	public int getWidth(){
		return this.width;
	}
	
	public int getHeight(){
		return this.height;
	}
}


