package com.example.cells.IO.Handlers;

//import android.view.KeyEvent;
import android.view.View;
import android.view.View.OnKeyListener;
import java.util.ArrayList;
import android.util.Log;
import java.lang.Integer;
import com.example.cells.IO.ObjectFactory;
import com.example.cells.IO.Pool;
import com.example.cells.IO.Input.KeyEvent;
import com.example.cells.IO.Input.TouchEvent;

public class KeyBoardHandler implements OnKeyListener {

	// android KEYCODE constants range between 0 and 127
	private boolean[] _pressedKeys = new boolean[128];

	private Pool<KeyEvent> _keyEventPool;
	private ObjectFactory<KeyEvent> _factory;
	private ArrayList<KeyEvent> _keyEventsBufferOne = new ArrayList<KeyEvent>();
	private ArrayList<KeyEvent> _keyEventsBufferTwo = new ArrayList<KeyEvent>();

	// logging and debugging
	private final String TAG = "KeyBoardHandler";

	public KeyBoardHandler(View view) {
		// create the specific factory for the pool
		_factory = new ObjectFactory<KeyEvent>() {
			public KeyEvent createObject() {
				return new KeyEvent();
			}
		};// ObjectFactory

		_keyEventPool = new Pool<KeyEvent>(_factory, 100);

		// set up this as the listener for the view
		view.setOnKeyListener(this);
		view.setFocusableInTouchMode(true);
		view.requestFocus();
	}

	// UI Thread as the view is passed into separate thread in CWSurfaceView
	public boolean onKey(View v, int keyCode, android.view.KeyEvent event) {
		boolean bool = false;

		if (event.getAction() == android.view.KeyEvent.ACTION_MULTIPLE) {
			return bool;
		}

		synchronized (this) {
			KeyEvent keyEvent = _keyEventPool.getObject();
			keyEvent.keyCode = keyCode; // This is the physical key pressed, not
										// Unicode char

			keyEvent.keyChar = (char) event.getUnicodeChar();

			if (event.getAction() == android.view.KeyEvent.ACTION_DOWN) {
				keyEvent.type = KeyEvent.KEY_DOWN;
				if (keyCode >= 0 && keyCode < 128) {
					_pressedKeys[keyCode] = true;
				}
			}// key was pressed

			if (event.getAction() == android.view.KeyEvent.ACTION_UP) {
				keyEvent.type = KeyEvent.KEY_UP;
				if (keyCode >= 0 && keyCode < 128) {
					_pressedKeys[keyCode] = false;
				}
			}// key not pressed
			_keyEventsBufferOne.add(keyEvent);
		}// sync
		return bool;
	}

	public boolean isKeyPressed(int keyCode) {

		if (keyCode >= 0 && keyCode < 128) {
			Log.v("isKeyPressed(int keyCode)", Integer.toString(keyCode));
			return _pressedKeys[keyCode];
		} else
			return false;

	}

	/**
	 * static KeyEvent objects contain keyCodes and keyChar info as well as
	 * KEY_UP, KEY_DOWN, etc ....
	 * These are static in the Input.java file.
	 */

	public ArrayList<KeyEvent> getKeyEvents() {
		synchronized (this) {// why would another thread be touching this?
			// add what you can back into the pool from the buffer
			int length = _keyEventsBufferTwo.size();
			for (int i = 0; i < length; i++) {
				_keyEventPool.returnToPool(_keyEventsBufferTwo.get(i));
			}
			// clear the buffer
			_keyEventsBufferTwo.clear();
			// add contents from other buffer into this one and flush other
			// buffer
			_keyEventsBufferTwo.addAll(_keyEventsBufferOne);
			_keyEventsBufferOne.clear();
			return _keyEventsBufferTwo;
		}
	}
}
