package com.example.cells.IO;

/**
 * Variables, methods and constructors which are declared protected in a superclass 
 * can be accessed only by the subclasses in other package or any class within the 
 * package of the protected members' class.
 **/

import java.util.List;
import java.util.ArrayList;
import com.example.cells.IO.Pool;
import com.example.cells.IO.ObjectFactory;
import com.example.cells.IO.Input.TouchEvent;
import android.util.Log;

public class Touch {
	
	private ObjectFactory factory;
	public boolean isTouched[];//for multitouch
	public Pool<TouchEvent> touchEvents;
	public ArrayList<TouchEvent> touchEventsBufferOne;
	public ArrayList<TouchEvent> touchEventsBufferTwo;
	//float scaleX, scaleY;
	int[] touchX, touchY;
	private String TAG = "TOUCH Class:";
	
	/***Inner Class*****/
	public static class TouchEvent {
		public static final int TOUCH_DOWN = 0;
		public static final int TOUCH_UP = 1;
		public static final int TOUCH_DRAGGED = 2;
		
		public int type;
		public int x,y;
		public int pointer;//corresponds to pointerId from event listener attached to view
	}
	
	public Touch(){ //float scaleX, float scaleY args to constructor
		factory = new ObjectFactory<TouchEvent>(){
			@Override
			public TouchEvent createObject() {
				return new TouchEvent();
			}
		};
		touchEvents = new Pool<TouchEvent>(factory, 100);
		this.isTouched = new boolean[20];
		this.touchX = new int[20];
		this.touchY = new int[20];
		touchEventsBufferOne = new ArrayList<TouchEvent>();
		touchEventsBufferTwo = new ArrayList<TouchEvent>();
	}
	
	public boolean isTouched(int pointerId){ //pointerId sustained through touchevents if a finger remains
		synchronized(this){
			if(pointerId < 0 || pointerId >=20){ //no errors, android only handles 10 events
				return false;
			}
			else{
				return isTouched[pointerId];
			}
		}
	}
	
	public int getTouchX(int pointerId){
		synchronized(this){
			if(pointerId < 0 || pointerId >=20){ 
				return 0;
			}
			else{
				return touchX[pointerId];
			}
		}
	}
	
	public int getTouchY(int pointerId){
		synchronized(this){
			if(pointerId < 0 || pointerId >=20){ 
				return 0;
			}
			else{
				return touchY[pointerId];
			}
		}
	}
	
	public ArrayList<TouchEvent> getTouchEvents(){
		synchronized(this){
			int length = touchEventsBufferTwo.size();
			for(int i=0; i<length; i++){
				touchEvents.returnToPool(touchEventsBufferTwo.get(i));
			}
				touchEventsBufferTwo.clear();
				Log.d(TAG, "Cleared touchEventsBufferTwo");
				touchEventsBufferTwo.addAll(touchEventsBufferOne);
				Log.d(TAG, "Added events from TouchEventBufferONE to touchEventsBufferTWO");
				touchEventsBufferOne.clear();
				Log.d(TAG, "Returning touchEventsBufferTwo contents");
			return touchEventsBufferTwo;
		}
	}
}
