package com.example.cells.Views;

import Game.CellGame;
import Game.Game;
import Screens.LoadingScreen;
import android.R.color;
import android.view.MotionEvent;
import android.view.SurfaceView;
import android.view.SurfaceHolder;
import android.view.View;
import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.Canvas;
import android.util.Log;
import com.example.cells.Graphics.Graphics;
import com.example.cells.IO.Touch;
import com.example.cells.IO.Touch.TouchEvent;


public class GameSurfaceView extends SurfaceView implements SurfaceHolder.Callback{
	private GameThread gameThread;
	private String TAG = "THREAD 1";
	public Graphics graphics;
	private Touch touch;
	private CellGame cellGame;
		
	public GameSurfaceView(Context context, Bitmap bitmap, int width, int height){
		super(context);
		setFocusable(true);
		graphics = new Graphics(context.getAssets(), width, height);
		touch = new Touch();
		cellGame = new CellGame(touch, graphics);
		
				
		//add callbacks for unlockAndPost
		getHolder().addCallback(this);
		this.gameThread = new GameThread(this, getHolder(), cellGame);//Loading Screen within game initialized
	}

	@Override
	public void surfaceChanged(SurfaceHolder holder, int format, int width,
			int height) {
		// TODO Auto-generated method stub
	}

	@Override
	public void surfaceCreated(SurfaceHolder holder) {
		// TODO Auto-generated method stub
		this.gameThread.setRun(true);
		Log.d(TAG, "thread run was set to TRUE");
		this.gameThread.start();
	}

	@Override
	public void surfaceDestroyed(SurfaceHolder holder) {
		// TODO Auto-generated method stub
		this.gameThread.setRun(false);
		Log.d(TAG, "thread run was set to FALSE");
		boolean retry=false; 
		
		while(retry){
			try{
				gameThread.join();
				retry=false;
			}catch(InterruptedException e){
				Log.d(TAG, e.toString());
			}
		}
	}
	
	//run from graphics thread
	@Override
	public void onDraw(Canvas canvas){
		super.onDraw(canvas);
		
	}

	@Override
	public boolean onTouchEvent(MotionEvent event) {
		 
		synchronized(this.touch){
			
		//move cell here
		logCatEvent(this,event);
		/** 0=ACTION_DOWN
		 *  1=ACTION_UP
		 *  2=ACTION_MOVE
		 */
		int action = event.getActionMasked();
		int pointerIndex = event.getActionIndex();
		int pointerId = event.getPointerId(pointerIndex);
		TouchEvent touchEvent;
		StringBuilder result = new StringBuilder(60);
		
			switch(action){
			case MotionEvent.ACTION_DOWN:
			case MotionEvent.ACTION_POINTER_DOWN:
				touchEvent = touch.touchEvents.getObject(); //grab touchEvent obj from pool
				touchEvent.type = TouchEvent.TOUCH_DOWN;
				touchEvent.pointer = pointerId;
				touch.isTouched[pointerId] = true;
				touch.touchEventsBufferOne.add(touchEvent);
				break;
			case MotionEvent.ACTION_UP:
			case MotionEvent.ACTION_POINTER_UP:
			case MotionEvent.ACTION_CANCEL:
				touchEvent = touch.touchEvents.getObject(); //grab touchEvent obj from pool
				touchEvent.type = TouchEvent.TOUCH_UP;
				touchEvent.pointer = pointerId;
				touch.isTouched[pointerId] = false;
				touchEvent.x = Math.round(event.getX(pointerIndex));
				touchEvent.y = Math.round(event.getY(pointerIndex));
				result.append("event.getX: "+(int)event.getX(pointerIndex)).append("  touchEvent.x: " + touchEvent.x).append(" touchEvent.y: " + touchEvent.y) ;
				Log.d("ONTOUCH", " ONTOUCH "+ result.toString() );
				
				touch.touchEventsBufferOne.add(touchEvent);
				break;
			case MotionEvent.ACTION_MOVE:
				touchEvent = touch.touchEvents.getObject(); //grab touchEvent obj from pool
				touchEvent.type = TouchEvent.TOUCH_DRAGGED;
				touchEvent.pointer = pointerId;
				touch.touchEventsBufferOne.add(touchEvent);
				break;
			}
		}//sync
		//gameScreen.setInput(Math.round(x), Math.round(y));
		return true;
		}
	
	public void logCatEvent(View v, MotionEvent event){
		StringBuilder result = new StringBuilder(60);
		result.append("ACTION: ").append(event.getAction()).append("\n");
		int ptrIndex=0;
		int numOfPointers=event.getPointerCount();
		result.append("Number of Pointers [fingers]: ").append(numOfPointers).append("\n");
		while(ptrIndex < numOfPointers){
			int pointerId = event.getPointerId(ptrIndex);
			result.append("Pointer Index: ").append(ptrIndex).append("\n");
			result.append("Pointer ID: ").append(pointerId).append("\n");
			result.append("LOCATION: X:").append(event.getX(ptrIndex)).append(" Y:").append(event.getY(ptrIndex));
			ptrIndex++;
		}
			
		Log.d(TAG, result.toString());
		
		StringBuilder detailedResult = new StringBuilder(60);
		//when touch has reached edge of physical screen
		detailedResult.append("Edge Flags: ").append(event.getEdgeFlags()).append("\n");
		detailedResult.append("Pressure: ").append(event.getPressure()).append("\n");
		detailedResult.append("Size: ").append(event.getSize()).append("\n");
		Log.d(TAG, detailedResult.toString());
	}
	
	public void surfaceViewResume(){
		
	}

	public void surfaceViewStop(){
		this.surfaceDestroyed(getHolder());
	}
}
